import 'package:flutter/material.dart';
import 'package:flutter_contacts/flutter_contacts.dart';
import 'package:intl/intl.dart';
import 'package:url_launcher/url_launcher.dart';
import 'package:dialer/features/contacts/contact_state.dart';
import 'package:dialer/widgets/username_color_generator.dart';
import 'package:dialer/widgets/color_darkener.dart';

class History {
  final Contact contact;
  final DateTime date;
  final String callType; // 'incoming' or 'outgoing'
  final String callStatus; // 'missed' or 'answered'
  final int attempts;

  History(
      this.contact,
      this.date,
      this.callType,
      this.callStatus,
      this.attempts,
      );
}

class HistoryPage extends StatefulWidget {
  const HistoryPage({Key? key}) : super(key: key);

  @override
  _HistoryPageState createState() => _HistoryPageState();
}

class _HistoryPageState extends State<HistoryPage> with SingleTickerProviderStateMixin {
  List<History> histories = [];
  bool loading = true;
  int? _expandedIndex;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    if (loading) {
      _buildHistories();
    }
  }

  Future<void> _buildHistories() async {
    final contactState = ContactState.of(context);
    if (contactState.loading) {
      // Wait for contacts to be loaded
      await Future.doWhile(() async {
        await Future.delayed(const Duration(milliseconds: 100));
        return contactState.loading;
      });
    }
    List<Contact> contacts = contactState.contacts;

    if (contacts.isEmpty) {
      setState(() {
        loading = false;
      });
      return;
    }

    setState(() {
      histories = List.generate(
        contacts.length >= 10 ? 10 : contacts.length,
            (index) => History(
          contacts[index],
          DateTime.now().subtract(Duration(hours: (index + 1) * 2)),
          index % 2 == 0 ? 'outgoing' : 'incoming',
          index % 3 == 0 ? 'missed' : 'answered',
          index % 3 + 1,
        ),
      );
      loading = false;
    });
  }

  List _buildGroupedList(List<History> historyList) {
    // Sort histories by date (most recent first)
    historyList.sort((a, b) => b.date.compareTo(a.date));

    final now = DateTime.now();
    final today = DateTime(now.year, now.month, now.day);
    final yesterday = today.subtract(const Duration(days: 1));

    List<History> todayHistories = [];
    List<History> yesterdayHistories = [];
    List<History> olderHistories = [];

    for (var history in historyList) {
      final callDate = DateTime(history.date.year, history.date.month, history.date.day);
      if (callDate == today) {
        todayHistories.add(history);
      } else if (callDate == yesterday) {
        yesterdayHistories.add(history);
      } else {
        olderHistories.add(history);
      }
    }

    // Combine them with headers
    final items = <dynamic>[];
    if (todayHistories.isNotEmpty) {
      items.add('Today');
      items.addAll(todayHistories);
    }
    if (yesterdayHistories.isNotEmpty) {
      items.add('Yesterday');
      items.addAll(yesterdayHistories);
    }
    if (olderHistories.isNotEmpty) {
      items.add('Older');
      items.addAll(olderHistories);
    }

    return items;
  }

  @override
  Widget build(BuildContext context) {
    final contactState = ContactState.of(context);

    if (loading || contactState.loading) {
      return Scaffold(
        backgroundColor: Colors.black,
        body: const Center(
          child: CircularProgressIndicator(),
        ),
      );
    }

    if (histories.isEmpty) {
      return Scaffold(
        backgroundColor: Colors.black,
        body: const Center(
          child: Text(
            'No call history available.',
            style: TextStyle(color: Colors.white),
          ),
        ),
      );
    }

    // Filter missed calls
    List<History> missedCalls = histories.where((h) => h.callStatus == 'missed').toList();

    final allItems = _buildGroupedList(histories);
    final missedItems = _buildGroupedList(missedCalls);

    return DefaultTabController(
      length: 2,
      child: Scaffold(
        backgroundColor: Colors.black,
        appBar: PreferredSize(
          preferredSize: const Size.fromHeight(kToolbarHeight),
          child: Container(
            color: Colors.black,
            child: const TabBar(
              tabs: [
                Tab(text: 'All Calls'),
                Tab(text: 'Missed Calls'),
              ],
              indicatorColor: Colors.white,
            ),
          ),
        ),
        body: TabBarView(
          children: [
            // All Calls
            _buildListView(allItems),
            // Missed Calls
            _buildListView(missedItems),
          ],
        ),
      ),
    );
  }

  Widget _buildListView(List items) {
    return ListView.builder(
      itemCount: items.length,
      itemBuilder: (context, index) {
        final item = items[index];

        if (item is String) {
          // This is a header item
          return Container(
            padding: const EdgeInsets.symmetric(vertical: 8, horizontal: 16),
            color: Colors.grey[900],
            child: Text(
              item,
              style: const TextStyle(
                color: Colors.white70,
                fontWeight: FontWeight.bold,
              ),
            ),
          );
        } else if (item is History) {
          final history = item;
          final contact = history.contact;
          final isExpanded = _expandedIndex == index;

          // Generate the avatar color
          Color avatarColor = generateColorFromName(contact.displayName);

          return Column(
            children: [
              ListTile(
                leading: (contact.thumbnail != null && contact.thumbnail!.isNotEmpty)
                    ? CircleAvatar(
                  backgroundImage: MemoryImage(contact.thumbnail!),
                )
                    : CircleAvatar(
                  backgroundColor: avatarColor,
                  child: Text(
                    contact.displayName.isNotEmpty
                        ? contact.displayName[0].toUpperCase()
                        : '?',
                    style: TextStyle(color: darken(avatarColor, 0.4)),
                  ),
                ),
                title: Text(
                  contact.displayName,
                  style: const TextStyle(color: Colors.white),
                ),
                subtitle: Text(
                  '${history.callType} - ${history.callStatus} - ${DateFormat('MMM dd, hh:mm a').format(history.date)}',
                  style: const TextStyle(color: Colors.grey),
                ),
                trailing: Row(
                  mainAxisSize: MainAxisSize.min,
                  children: [
                    Text(
                      '${history.attempts}x',
                      style: const TextStyle(color: Colors.white),
                    ),
                    IconButton(
                      icon: const Icon(Icons.phone, color: Colors.green),
                      onPressed: () async {
                        if (contact.phones.isNotEmpty) {
                          final Uri callUri =
                          Uri(scheme: 'tel', path: contact.phones.first.number);
                          if (await canLaunchUrl(callUri)) {
                            await launchUrl(callUri);
                          } else {
                            ScaffoldMessenger.of(context).showSnackBar(
                              const SnackBar(content: Text('Could not launch call')),
                            );
                          }
                        } else {
                          ScaffoldMessenger.of(context).showSnackBar(
                            const SnackBar(content: Text('Contact has no phone number')),
                          );
                        }
                      },
                    ),
                  ],
                ),
                onTap: () {
                  setState(() {
                    _expandedIndex = isExpanded ? null : index;
                  });
                },
              ),
              if (isExpanded)
                Container(
                  color: Colors.grey[850],
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceAround,
                    children: [
                      TextButton.icon(
                        onPressed: () async {
                          if (history.contact.phones.isNotEmpty) {
                            final Uri smsUri =
                            Uri(scheme: 'sms', path: history.contact.phones.first.number);
                            if (await canLaunchUrl(smsUri)) {
                              await launchUrl(smsUri);
                            } else {
                              ScaffoldMessenger.of(context).showSnackBar(
                                const SnackBar(content: Text('Could not send message')),
                              );
                            }
                          } else {
                            ScaffoldMessenger.of(context).showSnackBar(
                              const SnackBar(content: Text('Contact has no phone number')),
                            );
                          }
                        },
                        icon: const Icon(Icons.message, color: Colors.white),
                        label: const Text('Message', style: TextStyle(color: Colors.white)),
                      ),
                      TextButton.icon(
                        onPressed: () {
                          // Navigate to Call Details page
                          Navigator.push(
                            context,
                            MaterialPageRoute(
                              builder: (_) => CallDetailsPage(history: history),
                            ),
                          );
                        },
                        icon: const Icon(Icons.info, color: Colors.white),
                        label: const Text('Details', style: TextStyle(color: Colors.white)),
                      ),
                      TextButton.icon(
                        onPressed: () {
                          // Implement block number functionality
                          ScaffoldMessenger.of(context).showSnackBar(
                            const SnackBar(
                              content: Text('Number blocked (functionality not implemented)'),
                            ),
                          );
                        },
                        icon: const Icon(Icons.block, color: Colors.white),
                        label: const Text('Block', style: TextStyle(color: Colors.white)),
                      ),
                    ],
                  ),
                ),
            ],
          );
        }

        return const SizedBox.shrink();
      },
    );
  }
}

class CallDetailsPage extends StatelessWidget {
  final History history;

  const CallDetailsPage({Key? key, required this.history}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    final contact = history.contact;

    return Scaffold(
      backgroundColor: Colors.black,
      appBar: AppBar(
        title: const Text('Call Details'),
        backgroundColor: Colors.black,
      ),
      body: Padding(
        padding: const EdgeInsets.all(16.0),
        child: Column(
          children: [
            // Display Contact Name and Thumbnail
            Row(
              children: [
                (contact.thumbnail != null && contact.thumbnail!.isNotEmpty)
                    ? CircleAvatar(
                  backgroundImage: MemoryImage(contact.thumbnail!),
                  radius: 30,
                )
                    : CircleAvatar(
                  backgroundColor: Colors.grey[700],
                  radius: 30,
                  child: Text(
                    contact.displayName.isNotEmpty
                        ? contact.displayName[0].toUpperCase()
                        : '?',
                    style: const TextStyle(color: Colors.white),
                  ),
                ),
                const SizedBox(width: 16),
                Expanded(
                  child: Text(
                    contact.displayName,
                    style: const TextStyle(color: Colors.white, fontSize: 24),
                  ),
                ),
              ],
            ),
            const SizedBox(height: 24),

            // Display call type, status, date, attempts
            DetailRow(
              label: 'Call Type:',
              value: history.callType,
            ),
            DetailRow(
              label: 'Call Status:',
              value: history.callStatus,
            ),
            DetailRow(
              label: 'Date:',
              value: DateFormat('MMM dd, yyyy - hh:mm a').format(history.date),
            ),
            DetailRow(
              label: 'Attempts:',
              value: '${history.attempts}',
            ),

            const SizedBox(height: 24),

            // If you have more details like duration, contact number, etc.
            if (contact.phones.isNotEmpty)
              DetailRow(
                label: 'Number:',
                value: contact.phones.first.number,
              ),
          ],
        ),
      ),
    );
  }
}

class DetailRow extends StatelessWidget {
  final String label;
  final String value;

  const DetailRow({Key? key, required this.label, required this.value}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.symmetric(vertical: 8.0),
      child: Row(
        children: [
          Text(
            label,
            style: const TextStyle(color: Colors.white70, fontWeight: FontWeight.bold),
          ),
          const SizedBox(width: 8),
          Expanded(
            child: Text(
              value,
              style: const TextStyle(color: Colors.white),
              textAlign: TextAlign.right,
            ),
          ),
        ],
      ),
    );
  }
}
